@@ -1,4 +1,27 @@
-/* vi: ff=unix spell */
+/*
+Copyright (C) 1994-2007  Frans Slothouber, Jacco van Weert, Petteri Kettunen,
+Bernd Koesling, Thomas Aglassinger, Anthon Pang, Stefan Kost, David Druffner,
+Sasha Vasko, Kai Hofmann, Thierry Pierron, Friedrich Haase, and Gergely Budai.
+
+This file is part of ROBODoc
+
+ROBODoc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+*/
+
+
+
 /****h* ROBODoc/Analyser
  * NAME
  *   Analyser -- Functions to scan source and collect headers
@@ -7,7 +30,7 @@
  *   collect all the headers.
  *
  *****
- * $Id: analyser.c,v 1.72 2007/02/21 22:24:46 gumpu Exp $
+ * $Id: analyser.c,v 1.77 2009/03/16 21:41:23 gumpu Exp $
  */
 
 #include <stdlib.h>
@@ -137,7 +160,7 @@ static char        *Is_Tool(
 
     if ( *cur_char == '|' && *s )
     {
-        // Check if tool starts or ends
+        /* Check if tool starts or ends */
         if ( !strncmp( "tool ", s, 5 ) )
         {
             if ( *tool_active )
@@ -153,7 +176,7 @@ static char        *Is_Tool(
 
             return ( s + 5 );
         }
-        // Check if DOT starts or ends
+        /* Check if DOT starts or ends */
         if ( !strncmp( "dot ", s, 4 ) )
         {
             if ( *tool_active )
@@ -169,13 +192,13 @@ static char        *Is_Tool(
 
             return ( s + 4 );
         }
-        // Check for DOT file includes
+        /* Check for DOT file includes */
         else if ( !strncmp( "dotfile ", s, 8 ) && !*tool_active )
         {
             *itemkind = ITEM_LINE_DOT_FILE;
             return ( s + 8 );
         }
-        // Check for exec items
+        /* Check for exec items */
         else if ( !strncmp( "exec ", s, 5 ) && !*tool_active )
         {
             *itemkind = ITEM_LINE_EXEC;
@@ -246,59 +269,6 @@ void RB_Analyse_Document(
 
 /*****/
 
-#if 0
-/****f* Analyser/Check_Header_Start
- * FUNCION
- *   Sometimes a user makes a spelling mistake in the name of the first item.
- *   ROBODoc then ignores all the lines leading up to the second item,
- *   (which is the first item ROBODoc recognized).  This function
- *   checks for this condition and gives the user a warning.
- * SYNOPSIS
- */
-static void Check_Header_Start(
-    struct RB_header *arg_header,
-    int first_item )
-/*
- * INPUTS
- *   * arg_header -- the header to be checked.
- *   * first      -- the line on which the first item was found
- * RESULT
- *   A warning is given if the condition occured.
- * SOURCE
- */
-{
-    if ( first_item )
-    {
-        int                 i = 0;
-
-        for ( i = 0; i < first_item; ++i )
-        {
-            char               *c = arg_header->lines[i];
-
-            c = RB_Skip_Whitespace( c );
-            if ( RB_Has_Remark_Marker( c ) )
-            {
-                c = RB_Skip_Remark_Marker( c );
-                for ( ; *c; ++c )
-                {
-                    if ( !utf8_isspace( *c ) )
-                    {
-                        RB_Warning_Full( Get_Fullname
-                                         ( arg_header->owner->filename ),
-                                         arg_header->line_number,
-                                         "Header \"%s\" contains text before the fist item, "
-                                         "this might be caused by a misspelled item name.\n",
-                                         arg_header->name );
-                        return;
-                    }
-                }
-            }
-        }
-    }
-}
-
-/*******/
-#endif
 
 /****f* Analyser/Is_Empty_Line
  * FUNCTION
@@ -425,29 +395,6 @@ static int Is_List_Item_Continuation(
 /*****/
 
 
-#if 0
-/****f* Analyser/Is_End_of_List
- * FUNCTION
- *   Check... (TODO) 
- * INPUTS
- *   * arg_line --
- *   * indent  --
- * SOURCE
- */
-
-static int Is_End_of_List(
-    char *arg_line,
-    int indent )
-{
-    int                 this_indent = Get_Indent( arg_line );
-
-    return ( this_indent <= indent );
-}
-
-/******/
-#endif
-
-
 /****f* Analyser/Is_Start_List
  * FUNCTION
  *   Check... (TODO) 
@@ -650,43 +597,6 @@ static void Analyse_List(
 
 /******/
 
-// If (unused(this_function)) { delete(this_function) }
-#if 0
-/****f* Analyser/Dump_Item
- * FUNCTION
- *   Print debug information.
- */
-static void Dump_Item(
-    struct RB_Item *arg_item )
-/*
- * INPUTS
- *   * arg_item -- the item to be pretty-printed.
- * SOURCE
- */
-{
-    int                 i;
-
-    /* preformatted blocks */
-    for ( i = 0; i < arg_item->no_lines; i++ )
-    {
-        char               *line = arg_item->lines[i]->line;
-        int                 format = arg_item->lines[i]->format;
-
-        printf( "%04d ", i );
-        printf( ( format & RBILA_END_PARAGRAPH ) ? "p" : " " );
-        printf( ( format & RBILA_BEGIN_PARAGRAPH ) ? "P" : " " );
-        printf( ( format & RBILA_END_PRE ) ? "e" : " " );
-        printf( ( format & RBILA_BEGIN_PRE ) ? "E" : " " );
-        printf( ( format & RBILA_END_LIST ) ? "l" : " " );
-        printf( ( format & RBILA_BEGIN_LIST ) ? "L" : " " );
-        printf( ( format & RBILA_END_LIST_ITEM ) ? "i" : " " );
-        printf( ( format & RBILA_BEGIN_LIST_ITEM ) ? "I" : " " );
-        printf( " (%s)\n", line );
-    }
-}
-
-/*******/
-#endif
 
 /****f* Analyser/Preformat_All
  * FUNCTION
@@ -754,12 +664,16 @@ static void Preformat_All(
                      * it, otherwise a begin and end pre will be
                      * generated, in the wrong order, on the same line in the output.
                      */
-                    if ( arg_item->lines[i]->format & RBILA_BEGIN_PRE ) {
+                    if ( arg_item->lines[i]->format & RBILA_BEGIN_PRE )
+                    {
                         arg_item->lines[i]->format &= ~( RBILA_BEGIN_PRE );
-                    } else {
+                    }
+                    else
+                    {
                         arg_item->lines[i]->format |= RBILA_END_PRE;
                     }
-                    arg_item->lines[i]->format |= ( source ? RBILA_END_SOURCE : 0 );
+                    arg_item->lines[i]->format |=
+                        ( source ? RBILA_END_SOURCE : 0 );
                 }
             }
         }
@@ -1031,38 +945,38 @@ static void Analyse_Item_Format(
  * SOURCE
  */
 {
-    // If item is not empty
+    /* If item is not empty */
     if ( arg_item->no_lines )
     {
-        // If it is a SOURCE item
+        /* If it is a SOURCE item */
         if ( Works_Like_SourceItem( arg_item->type ) )
         {
-            // Preformat it like a SOURCE item
+            /* Preformat it like a SOURCE item */
             Preformat_All( arg_item, TRUE );
         }
-        // Check if we have to analyse this item
+        /* Check if we have to analyse this item */
         else if ( ( course_of_action.do_nopre
                     || Is_Format_Item( arg_item->type ) )
                   && !Is_Preformatted_Item( arg_item->type ) )
         {
-            // analyse item
+            /* analyse item */
             int                 indent = Analyse_Indentation( arg_item );
 
             Analyse_List( arg_item, indent );
             Analyse_Preformatted( arg_item, indent );
             Analyse_Paragraphs( arg_item );
         }
-        // If none of above, preformat item
+        /* If none of above, preformat item */
         else
         {
-            // Preformat it
+            /* Preformat it */
             Preformat_All( arg_item, FALSE );
         }
     }
-    // Item is empty
+    /* Item is empty */
     else
     {
-        // Do nothing
+        /* Do nothing */
     }
 }
 
@@ -1083,14 +997,14 @@ static int Trim_Empty_Item_Begin_Lines(
         /* We skip the first line after the source item, if
          * it an remark end marker -- such as '*)'
          */
-        c = arg_header->lines[current_index];
+        c = arg_header->lines[current_index].line;
         if ( RB_Is_Remark_End_Marker( c ) )
         {
             c = RB_Skip_Remark_End_Marker( c );
             c = RB_Skip_Whitespace( c );
             if ( *c != '\0' )
             {
-                c = arg_header->lines[current_index];
+                c = arg_header->lines[current_index].line;
                 RB_Warning( "text following a remark end marker:\n%s\n", c );
             }
             ++current_index;
@@ -1105,7 +1019,7 @@ static int Trim_Empty_Item_Begin_Lines(
     {
         do
         {
-            c = arg_header->lines[current_index];
+            c = arg_header->lines[current_index].line;
             c = RB_Skip_Whitespace( c );
             if ( RB_Has_Remark_Marker( c ) )
             {
@@ -1134,14 +1048,14 @@ static int Trim_Empty_Item_End_Lines(
 
     if ( Works_Like_SourceItem( arg_item->type ) )
     {
-        c = arg_header->lines[current_index];
+        c = arg_header->lines[current_index].line;
         if ( RB_Is_Remark_Begin_Marker( c ) )
         {
             c = RB_Skip_Remark_Begin_Marker( c );
             c = RB_Skip_Whitespace( c );
             if ( *c != '\0' )
             {
-                c = arg_header->lines[current_index];
+                c = arg_header->lines[current_index].line;
                 RB_Warning( "text following a remark begin marker:\n%s\n",
                             c );
             }
@@ -1151,7 +1065,7 @@ static int Trim_Empty_Item_End_Lines(
 
     do
     {
-        c = arg_header->lines[current_index];
+        c = arg_header->lines[current_index].line;
         c = RB_Skip_Whitespace( c );
         if ( RB_Has_Remark_Marker( c ) )
         {
@@ -1232,14 +1146,6 @@ static void Copy_Lines_To_Item(
     struct RB_header *arg_header,
     struct RB_Item *arg_item )
 {
-#if 0
-    printf( "%d\n%d\n%s\n%s\n",
-            arg_item->begin_index,
-            arg_item->end_index,
-            arg_header->lines[arg_item->begin_index],
-            arg_header->lines[arg_item->end_index] );
-#endif
-
     Trim_Empty_Item_Lines( arg_header, arg_item );
 
     if ( arg_item->no_lines > 0 )
@@ -1247,7 +1153,7 @@ static void Copy_Lines_To_Item(
         int                 i = 0;
         int                 j = 0;
         struct RB_Item_Line *itemline = NULL;
-        int                 tool_active = 0;    // Shows wether we are inside a tool body
+        int                 tool_active = 0;    /* Shows wether we are inside a tool body */
 
         /* Allocate enough memory for all the lines, plus one
          * extra line
@@ -1266,7 +1172,7 @@ static void Copy_Lines_To_Item(
         for ( i = 0; i < arg_item->no_lines - 1; ++i )
         {
             char               *c =
-                arg_header->lines[arg_item->begin_index + i];
+                arg_header->lines[arg_item->begin_index + i].line;
             /* TODO should be a Create_ItemLine() */
             itemline = malloc( sizeof( struct RB_Item_Line ) );
             if ( !itemline )
@@ -1276,7 +1182,7 @@ static void Copy_Lines_To_Item(
 
             c = ExpandTab( c );
             c = RB_Skip_Whitespace( c );
-            // Lines with remark marker
+            /* Lines with remark marker */
             if ( RB_Has_Remark_Marker( c )
                  && !Works_Like_SourceItem( arg_item->type ) )
             {
@@ -1288,37 +1194,38 @@ static void Copy_Lines_To_Item(
                 c2 = RB_Skip_Whitespace( c );
                 if ( *c2 )
                 {
-                    // Check wether a tool starts or ends
+                    /* Check wether a tool starts or ends */
                     if ( ( c3 = Is_Tool( c2, &item_kind, &tool_active ) ) )
                     {
                         itemline->kind = item_kind;
                         c = c3;
                     }
-                    // If we have an active tool, copy the body lines
+                    /* If we have an active tool, copy the body lines
+                     */
                     else if ( tool_active )
                     {
                         itemline->kind = ITEM_LINE_TOOL_BODY;
-                        c++;    // Skip space after the remark marker
+                        c++;    /* Skip space after the remark marker */
                     }
-                    // Check for pipes
+                    /* Check for pipes */
                     else if ( ( c3 = Is_Pipe_Marker( c2, &pipe_mode ) ) )
                     {
                         itemline->kind = ITEM_LINE_PIPE;
                         itemline->pipe_mode = pipe_mode;
                         c = c3;
                     }
-                    // Plain Items ...
+                    /* Plain Items ... */
                     else
                     {
                         itemline->kind = ITEM_LINE_PLAIN;
                     }
                 }
-                // Empty lines with remark markers and active tool
+                /* Empty lines with remark markers and active tool */
                 else if ( tool_active )
                 {
                     itemline->kind = ITEM_LINE_TOOL_BODY;
                 }
-                // Plain empty lines with remark markers...
+                /* Plain empty lines with remark markers... */
                 else
                 {
                     itemline->kind = ITEM_LINE_PLAIN;
@@ -1330,15 +1237,25 @@ static void Copy_Lines_To_Item(
                 /* The is raw code, so we do not want to have the
                  * whitespace stripped of
                  */
-                c = arg_header->lines[arg_item->begin_index + i];
+                c = arg_header->lines[arg_item->begin_index + i].line;
                 c = ExpandTab( c );
             }
 
+            /* Copy source like items */
             if ( ( !Works_Like_SourceItem( arg_item->type ) &&
                    ( itemline->kind != ITEM_LINE_RAW ) ) ||
                  Works_Like_SourceItem( arg_item->type ) )
             {
+                /* Copy item line */
                 itemline->line = RB_StrDup( c );
+                /* Copy original source line number */
+                itemline->line_number =
+                    arg_header->lines[arg_item->begin_index + i].line_number;
+                /* Update the maximum source line number */
+                if ( itemline->line_number > arg_item->max_line_number )
+                {
+                    arg_item->max_line_number = itemline->line_number;
+                }
                 itemline->format = 0;
                 arg_item->lines[j] = itemline;
                 ++j;
@@ -1386,6 +1303,30 @@ static void Copy_Lines_To_Item(
     }
 }
 
+/*RZ*/
+#ifdef EXPERIMENTAL
+static void Add_Lines_To_Item(
+        struct RB_Item *arg_item,
+        char *c )
+{
+    struct RB_Item_Line *itemline = NULL;
+
+    arg_item->no_lines++;
+    arg_item->lines = realloc( arg_item->lines, arg_item->no_lines * sizeof( struct RB_Item_Line * ) );
+    itemline = malloc( sizeof( struct RB_Item_Line ) );
+    if ( !itemline || !arg_item->lines) {
+        RB_Panic( "Out of memory! %s (2)\n", "Add_Lines_To_Item" );
+    }
+    itemline->line = RB_StrDup( c );
+    itemline->kind = ITEM_LINE_PLAIN;
+    itemline->format = 0;
+    if ( arg_item->no_lines > 1 ) {
+        arg_item->lines[arg_item->no_lines-1] = arg_item->lines[arg_item->no_lines-2];
+    }
+    arg_item->lines[arg_item->no_lines-2] = itemline;
+}
+#endif
+
 
 /****f* Analyser/RB_Analyse_Items
  * FUNCTION
@@ -1403,13 +1344,18 @@ static int Analyse_Items(
     enum ItemType       item_type = NO_ITEM;
     struct RB_Item     *new_item;
     struct RB_Item     *cur_item;
-
+#ifdef EXPERIMENTAL
+/*RZ*/
+int isSource=0;
+struct RB_Item *func_item = NULL;
+char *c;
+#endif
     RB_Item_Lock_Reset(  );
 
     /* find the first item */
     for ( line_nr = 0; line_nr < arg_header->no_lines; ++line_nr )
     {
-        item_type = RB_Is_ItemName( arg_header->lines[line_nr] );
+        item_type = RB_Is_ItemName( arg_header->lines[line_nr].line );
         if ( item_type != NO_ITEM )
         {
             break;
@@ -1421,6 +1367,13 @@ static int Analyse_Items(
     {
         new_item = RB_Create_Item( item_type );
         new_item->begin_index = line_nr;
+#ifdef EXPERIMENTAL
+/*RZ*/
+if (strcmp(configuration.items.names[item_type],"FUNCTION")==0) {
+  func_item =  new_item;
+}
+isSource= !strcmp(configuration.items.names[item_type],"SOURCE");
+#endif
 
         /* Add the item to the end of the list of items. */
         if ( arg_header->items )
@@ -1439,7 +1392,20 @@ static int Analyse_Items(
         /* Find the next item */
         for ( ++line_nr; line_nr < arg_header->no_lines; ++line_nr )
         {
-            item_type = RB_Is_ItemName( arg_header->lines[line_nr] );
+#ifdef EXPERIMENTAL
+/*RZ*/
+if (isSource && func_item) {
+  c = RB_Skip_Whitespace (arg_header->lines[line_nr].line);
+  if (strlen(c) > 2) {
+    if (c[0]=='#') {/* shell comments */
+      Add_Lines_To_Item( func_item, &c[1]);
+    } else if (c[0]=='/' && c[1]=='/') {/* C-comments */
+      Add_Lines_To_Item( func_item, &c[2]);
+    }
+  }
+}
+#endif
+            item_type = RB_Is_ItemName( arg_header->lines[line_nr].line );
             if ( item_type != NO_ITEM )
             {
                 break;
@@ -1868,12 +1834,11 @@ static int Find_End_Marker(
     int                 found = FALSE;
     unsigned int        no_lines = 0;
     unsigned int        max_no_lines = 10;
-    char              **lines = NULL;
-    char              **new_lines = NULL;
+    struct RB_header_lines *lines = NULL;
     char               *dummy;
 
-    lines = ( char ** ) calloc( max_no_lines, sizeof( char * ) );
-    if ( !lines )
+    lines = malloc( max_no_lines * sizeof( struct RB_header_lines ) );
+    if ( lines == NULL )
     {
         RB_Panic( "Out of memory! %s()\n", "Find_End_Marker" );
     }
@@ -1907,19 +1872,23 @@ static int Find_End_Marker(
             assert( line[n - 1] == '\n' );
             /* Strip CR */
             line[n - 1] = '\0';
-            lines[no_lines] = line;
+            /* Copy the line itself */
+            lines[no_lines].line = line;
+            /* and also save the original source line number */
+            lines[no_lines].line_number = line_number;
             ++no_lines;
             if ( no_lines == max_no_lines )
             {
                 max_no_lines *= 2;
-                new_lines = realloc( lines, max_no_lines * sizeof( char * ) );
+                lines =
+                    realloc( lines,
+                             max_no_lines *
+                             sizeof( struct RB_header_lines ) );
 
-                if ( !new_lines )
+                if ( lines == NULL )
                 {
                     RB_Panic( "Out of memory! %s()\n", "Find_End_Marker" );
                 }
-
-                lines = new_lines;
             }
         }
     }
@@ -1974,14 +1943,14 @@ static int RB_WordWithSpacesLen(
 
     for ( len = 0; ( ( c = *str ) != '\0' ) && ( c != '\n' ); ++str, ++len )
     {
-        // Look for header truncating characters
+        /* Look for header truncating characters */
         if ( Find_Parameter_Char( &( configuration.header_separate_chars ),
                                   c ) != NULL
              ||
              Find_Parameter_Char( &( configuration.header_ignore_chars ),
                                   c ) != NULL )
         {
-            // and exit loop if any found
+            /* and exit loop if any found */
             break;
         }
     }
@@ -2040,8 +2009,8 @@ static int Find_Header_Name(
                 RB_FreeLineBuffer(  );
                 myLine = RB_ReadWholeLine( fh, line_buffer, &readChars );
                 line_number++;
-                for ( cur_char = myLine; *cur_char && !utf8_isalpha( *cur_char );
-                      cur_char++ );
+                for ( cur_char = myLine;
+                      *cur_char && !utf8_isalpha( *cur_char ); cur_char++ );
             }
         }
     }
