@@ -1,3 +1,25 @@
+/*
+Copyright (C) 1994-2007  Frans Slothouber, Jacco van Weert, Petteri Kettunen,
+Bernd Koesling, Thomas Aglassinger, Anthon Pang, Stefan Kost, David Druffner,
+Sasha Vasko, Kai Hofmann, Thierry Pierron, Friedrich Haase, and Gergely Budai.
+
+This file is part of ROBODoc
+
+ROBODoc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+*/
+
 
 #include <string.h>
 #include <stdlib.h>
@@ -7,136 +29,8 @@
 #include <errno.h>
 #include <ctype.h>
 
-#if defined(__MINGW32__) || defined( RB_MSVC )
-
-#include "items.h"
-#include "headers.h"
-#include "robodoc.h"
-
-/* Set of dummy functions just to make it compile. */
-char               *RB_TROFF_Get_Default_Extension(
-    void )
-{
-    return ".3";                /* Section 3 of manual */
-}
-
-void RB_TROFF_Generate_Doc_End(
-    FILE *dest_doc,
-    char *name )
-{
-    USE( dest_doc );
-    USE( name );
-}
-
-FILE               *RB_TROFF_Generate_Header_Start(
-    FILE *dest_doc,
-    struct RB_header *cur_header )
-{
-    USE( dest_doc );
-    USE( cur_header );
-    return NULL;
-}
-
-void RB_TROFF_Generate_Header_End(
-    FILE *dest_doc,
-    struct RB_header *cur_header )
-{
-    USE( dest_doc );
-    USE( cur_header );
-}
-
-
-void RB_TROFF_Generate_Char(
-    FILE *dest_doc,
-    int c )
-{
-    USE( dest_doc );
-    USE( c );
-}
-
-void RB_TROFF_Generate_False_Link(
-    FILE *out,
-    char *name )
-{
-    USE( out );
-    USE( name );
-
-}
-
-void RB_TROFF_Generate_Item_Name(
-    FILE *out,
-    char *name,
-    int pre )
-{
-    USE( out );
-    USE( name );
-    USE( pre );
-}
-
-void RB_TROFF_Start_New_Line(
-    FILE *out )
-{
-    USE( out );
-}
-
-int RB_TROFF_Generate_Extra(
-    FILE *out,
-    enum ItemType item_type,
-    char *str )
-{
-    USE( out );
-    USE( item_type );
-    USE( str );
-    return -1;
-}
-
-void RB_TROFF_Set_Param(
-    char *compress,
-    char *section )
-{
-    USE( compress );
-    USE( section );
-}
-
-void TROFF_Generate_Begin_List_Item(
-    FILE *out )
-{
-    USE( out );
-}
-
-void TROFF_Generate_End_List_Item(
-    FILE *out )
-{
-    USE( out );
-}
-
-void TROFF_Generate_Begin_Preformatted(
-    FILE *out )
-{
-    USE( out );
-}
-
-void TROFF_Generate_End_Preformatted(
-    FILE *out )
-{
-    USE( out );
-}
-
-void TROFF_Generate_End_Paragraph(
-    FILE *out )
-{
-    USE( out );
-}
-
-void TROFF_Generate_Begin_Paragraph(
-    FILE *out )
-{
-    USE( out );
-}
-
-#else
-
 #include <sys/param.h>
+#include <sys/unistd.h>
 
 #include "troff_generator.h"
 #include "util.h"
@@ -182,7 +76,8 @@ void RB_TROFF_Generate_Doc_End(
     FILE *dest_doc,
     char *name )
 {
-
+    USE( dest_doc );
+    USE( name );
 }
 
 static FILE        *open_output_file(
@@ -258,9 +153,10 @@ FILE               *RB_TROFF_Generate_Header_Start(
     struct RB_header *cur_header )
 {
     static char        *manpage = NULL;
+    int                sec_item;
 
     if ( manpage == NULL )
-        manpage = malloc( MAXPATHLEN );
+        manpage = RB_malloc( MAXPATHLEN );
 
     if ( cur_header->name )
     {
@@ -269,10 +165,45 @@ FILE               *RB_TROFF_Generate_Header_Start(
         char               *name = basename( cur_header->name );
         char               *path = basename( file );
         int                 len, i;
+        struct RB_Item      *item;
+        char                *section = NULL;
+
+        /* Search for SECTION header and use this section for man pages */
+        sec_item = RB_Get_Item_Type("SECTION");
+        for( item = cur_header->items; item ; item = item->next) {
+            if( item->type == sec_item ) {
+                /* Item found, search for nonempty line */
+                for(i = 0 ; i < item->no_lines; i++ ) {
+                    section = item->lines[i]->line;
+                    while( *section && utf8_isspace(*section) ) section++;
+                    if( *section ) {
+                        /* nonempty line found, break */
+                        break;
+                    }
+                }
+                break;
+            }
+        }
 
         len = path - file;
         memcpy( manpage, file, len );
-        sprintf( manpage + len, "%s.%s%s", name, man_section, compress_ext );
+
+        /* we found a section header, strip at next whitespace */
+        if( section && *section ) {
+            char *to, *ptr;
+            ptr = RB_malloc( strlen(section) + 1 );
+            to = ptr;
+            while( *section && !utf8_isspace(*section) ) {
+                *to++ = *section++;
+            }
+            *to++ = 0;
+            section = ptr;
+        }
+        else {
+	    section = man_section;
+        }
+
+        sprintf( manpage + len, "%s.%s%s", name, section, compress_ext );
 
         dest_doc = open_output_file( manpage );
         if ( dest_doc == NULL )
@@ -282,18 +213,22 @@ FILE               *RB_TROFF_Generate_Header_Start(
 
         RB_Say( "+ Generating man page \"%s\"\n", SAY_INFO, manpage );
 
-        /* Check for aliases */
+        /* Check for aliases if we have symlink() */
+#ifdef HAVE_SYMLINK
         for ( i = 1; i < cur_header->no_names; i++ )
         {
             char               *base = basename( cur_header->names[i] );
             char               *buf = strchr( manpage, 0 ) + 1;
 
             memcpy( buf, file, len );
-            sprintf( buf + len, "%s.%s%s", base, man_section, compress_ext );
+            sprintf( buf + len, "%s.%s%s", base, section, compress_ext );
             unlink( buf );
             symlink( basename( manpage ), buf );
             RB_Say( "+ Linked with \"%s\"\n", SAY_INFO, buf );
         }
+#else
+        USE( i );
+#endif
 
         /* Append document type and title */
         fprintf( dest_doc, ".\\\" Source: %s\n",
@@ -301,7 +236,7 @@ FILE               *RB_TROFF_Generate_Header_Start(
 
         write_comment( dest_doc, COMMENT_ROBODOC COMMENT_COPYRIGHT );
 
-        fprintf( dest_doc, ".TH %s %s \"", name, man_section );
+        fprintf( dest_doc, ".TH %s %s \"", name, section );
         {
             time_t              ttp;
             char                timeBuffer[255];
@@ -318,13 +253,16 @@ FILE               *RB_TROFF_Generate_Header_Start(
             }
             else
                 len = strlen( path = name );
-            if ( len >= sizeof timeBuffer )
+            if ( len >= (signed)sizeof timeBuffer )
                 len = sizeof timeBuffer - 1;
             memcpy( timeBuffer, path, len );
             timeBuffer[len] = 0;
             fprintf( dest_doc, "\" %s \"%s Reference\"\n", timeBuffer,
                      timeBuffer );
         }
+        if( section != man_section ) {
+             free(section);
+        }
     }
     return dest_doc;
 }
@@ -339,6 +277,7 @@ void RB_TROFF_Generate_Header_End(
     FILE *dest_doc,
     struct RB_header *cur_header )
 {
+	USE( cur_header );
     fclose( dest_doc );
 }
 
@@ -386,6 +325,7 @@ void RB_TROFF_Generate_False_Link(
     if ( in_list )
         fprintf( out, " \\- " );
     in_list = 0;
+    skip_space = 0;
 }
 
 void RB_TROFF_Generate_Item_Name(
@@ -402,6 +342,7 @@ void RB_TROFF_Generate_Item_Name(
     {
         fprintf( out, ".nf\n" );
         skippre = -1;
+        skippre = 0; 
     }
     else
         skippre = 0;
@@ -426,6 +367,8 @@ int RB_TROFF_Generate_Extra(
     enum ItemType item_type,
     char *str )
 {
+	USE( item_type );
+
     if ( end_of_line == 0 )
         return -1;
     end_of_line = 0;
@@ -474,15 +417,56 @@ int RB_TROFF_Generate_Extra(
             strcpy( p, p + 2 );
             skip = item;
             *item = ' ';
+            in_list = 2;
         }
         else
         {
+            char m = *p;
             if ( !utf8_isspace( p[1] ) )
                 return -1;
-            fprintf( out, ".IP -\n" );
+	    if( index(p, ':') != NULL ) {
+                fprintf( out, ".IP \"" );
+                if( m == '*' ) fprintf( out, "\\fB" );
+                if( m == 'o' ) fprintf( out, "\\fI");
+                /* p[0] = - / o / *  and p[1] isspace, so we can incr. p */
+                p += 2;
+                while( (*p) && (*p != ':') ) {
+                    fputc(*p, out);
+                    p++;
+                    skip++;
+                }
+		while( *p != ':' ) {
+		    p++;
+		    skip++;
+		}
+		p++;
+		skip++;
+		while( utf8_isspace( *p ) ) {
+		    p++;
+		    skip++;
+		}
+                if( m == '*' ) fprintf( out, "\\fR");
+                if( m == 'o' ) fprintf( out, "\\fR");
+                fprintf( out, "\"\n");
+                skip++;
+	    }
+	    else {
+                switch( m ) {
+                case '*':
+                    fprintf( out, ".IP *\n" );
+                    break;
+                case 'o':
+                    fprintf( out, ".IP o\n" );
+                    break;
+                case '-':
+                default:
+                    fprintf( out, ".IP -\n" );
+                    break;
+                }
+	    }
             skip++;
+            in_list = 2;
         }
-        in_list = 2;
         skip_space = 1;
         return skip > str ? skip - str - 1 : -1;
     }
@@ -522,38 +506,60 @@ void RB_TROFF_Set_Param(
 void TROFF_Generate_Begin_List_Item(
     FILE *out )
 {
+    USE( out );
 /*	fprintf( out, ".IP -\n" ); */
 }
 
 void TROFF_Generate_End_List_Item(
     FILE *out )
 {
+    USE( out );
 /*	fprintf( out, ".PP\n" ); */
 }
 
 void TROFF_Generate_Begin_Preformatted(
     FILE *out )
 {
+    USE( out );
 /*	fprintf( out, ".nf\n" ); */
 }
 
 void TROFF_Generate_End_Preformatted(
     FILE *out )
 {
+    USE( out );
 /*	fprintf( out, ".fi\n" ); */
 }
 
 void TROFF_Generate_End_Paragraph(
     FILE *out )
 {
+    USE( out );
 /*	fputc( '\n', out ); */
 }
 
 void TROFF_Generate_Begin_Paragraph(
     FILE *out )
 {
+    USE( out );
 /*	fprintf( out, ".PP\n" ); */
 }
 
-#endif
-
+/****f* TROFF_Generator/RB_TROFF_Generate_Item_Line_Number
+ * FUNCTION
+ *   Generate line numbers for SOURCE like items
+ * SYNOPSIS
+ */
+void RB_TROFF_Generate_Item_Line_Number(
+    FILE *dest_doc,
+    char *line_number_string )
+/*
+ * INPUTS
+ *   o dest_doc           -- the file to write to.
+ *   o line_number_string -- the line number as string.
+ * SOURCE
+ */
+{
+    fprintf( dest_doc, line_number_string );
+}
+/******/
